#! perl

use strict;
use warnings;
use utf8;

package ChordPro::Wx;

# Documentation only.

1;

=head1 NAME

wxchordpro - a simple Wx-based GUI wrapper for ChordPro

=head1 SYNOPSIS

  wxchordpro [ ChordPro file ]

=head1 DESCRIPTION

B<wxchordpro> is a GUI wrapper for the ChordPro program. It allows
opening of files, make changes, and preview (optionally print) the
formatted result.

For more information about the ChordPro file format, see
L<https://www.chordpro.org>.

For more information about ChordPro program, see L<ChordPro>.

=cut

